<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2016 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */

defined('_JEXEC') or die;

/**
 * ChannelFactory class.
 *
 * Factory to create channel classes.
 * This is the central point to get and handle channel classes. Also all needed files are included here.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class ChannelFactory
{
	private $input = null;

	private $channels_filter = null;

	private $active_channels_filter = null;

	private $allowed_channels = null;

	/**
	 * ChannelFactory
	 *
	 */
	protected function __construct()
	{
		if (F0FPlatform::getInstance()->isCli())
		{
			$this->input = new JInputCli;
		}
		else
		{
			$this->input = new JInput;
		}

		$this->channels_filter = $this->input->get('channels_filter', '', 'string');
		$this->active_channels_filter = (!empty($this->channels_filter));

		if ($this->active_channels_filter)
		{
			$this->allowed_channels = TextUtil::listToArray($this->channels_filter);
			$this->active_channels_filter = (!empty($this->allowed_channels));
		}
	}

	/**
	 * getInstance
	 *
	 * @return	object.
	 */
	public static function getInstance()
	{
		return new ChannelFactory;
	}

	/**
	 * createChannel
	 *
	 * @param   F0FTable  &$channel      Param
	 * @param   bool      $channel_freq  Param
	 *
	 * @return	object
	 */
	protected function createChannel(&$channel, $channel_freq = false)
	{
		$channeltype = F0FModel::getTmpInstance('Channeltypes', 'AutoTweetModel');
		$classname = $channeltype->getChannelClass($channel->channeltype_id);
		JLoader::load($classname);

		if (!class_exists($classname))
		{
			throw new Exception(
				JText::sprintf('COM_AUTOTWEET_UNABLETO', 'channel type ('
						. $channel->channeltype_id . ')')
			);
		}

		if ((AUTOTWEETNG_JOOCIAL) && ($channel_freq))
		{
			$xtform = new JRegistry;
			$xtform->loadString($channel->params);
			$channel_freq_mhdmd = $xtform->get('channel_freq_mhdmd');

			if ( (!empty($channel_freq_mhdmd)) && ($channel_freq_mhdmd != '* * * * *') )
			{
				$automators = F0FModel::getTmpInstance('Automators', 'AutoTweetModel');
				$key = 'channel-' . $channel->id;

				if (!$automators->lastRunCheckFreqMhdmd($key, $channel_freq_mhdmd))
				{
					$logger = AutotweetLogger::getInstance();
					$logger->log(JLog::INFO, 'getChannel: channel ' . $channel->id . ' has been filtered out. Filter mhdmd: ' . $channel_freq_mhdmd);

					return null;
				}
			}
		}

		$c = new $classname($channel);

		return $c;
	}

	/**
	 * getChannels
	 *
	 * @param   string  $author  Param
	 *
	 * @return	array
	 */
	public function getChannels($author)
	{
		$channels = F0FModel::getTmpInstance('Channels', 'AutoTweetModel');
		$channels->set('published', true);
		$channels->set('scope', 'S');
		$channels->set('filter_order', 'ordering');
		$channels->set('filter_order_Dir', 'ASC');

		$list = $channels->getItemList(true);

		if (!empty($author))
		{
			$user_id = JUserHelper::getUserId($author);

			if ($user_id)
			{
				$userChannels = F0FModel::getTmpInstance('Channels', 'AutoTweetModel');
				$userChannels->set('published', true);
				$userChannels->set('scope', 'U');
				$userChannels->set('created_by', $user_id);
				$userChannels->set('filter_order', 'ordering');
				$userChannels->set('filter_order_Dir', 'ASC');

				$userList = $userChannels->getItemList(true);
				$list = array_merge($list, $userList);
			}
		}

		$channels = array();
		$logger = AutotweetLogger::getInstance();

		foreach ($list as $channel)
		{
			if ( ($this->active_channels_filter) && ($channel->scope == 'S') )
			{
				if (in_array($channel->id, $this->allowed_channels))
				{
					$c = self::createChannel($channel, true);

					if ($c)
					{
						$channels[$channel->id] = $c;
					}
				}
				else
				{
					$logger->log(JLog::INFO, 'getChannels: channel ' . $channel->id . ' has been filtered out. Filter: ', $this->allowed_channels);
				}
			}
			else
			{
				$c = self::createChannel($channel, true);

				if ($c)
				{
					$channels[$channel->id] = $c;
				}
			}
		}

		$channels_ids = array_keys($channels);
		$logger->log(JLog::INFO, 'ChannelFactory getChannels user=' . $author, $channels_ids);

		return $channels;
	}

	/**
	 * getChannel
	 *
	 * @param   int  $id  Param
	 *
	 * @return	object.
	 */
	public function getChannel($id)
	{
		$channels = F0FModel::getTmpInstance('Channels', 'AutoTweetModel');
		$channel = $channels->getItem($id);

		return self::createChannel($channel);
	}
}
