<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2016 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */

defined('_JEXEC') or die;

/**
 * FacebookChannelHelper class.
 * AutoTweet Facebook channel for wall posts.
 * Posts to the wall of profiles, groups, pages, events
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class FacebookChannelHelper extends FacebookBaseChannelHelper
{
	/**
	 * sendMessage.
	 *
	 * @param   string  $message  Params
	 * @param   object  $data     Params
	 *
	 * @return	booleans
	 */
	public function sendMessage($message, $data)
	{
		// Simulated
		if ($this->channel->params->get('use_own_api') == 0)
		{
			return $this->testMessage();
		}

		$isUserProfile = $this->isUserProfile();
		$enabledOpenGraphFeatures = $this->channel->params->get('open_graph_features');

		if (($enabledOpenGraphFeatures) && ($isUserProfile))
		{
			return $this->sendFacebookOG($message, $data->title, $data->fulltext, $data->url, $data->org_url, $data->image_url, $this->getMediaMode(), $data);
		}

		$name 			= $data->title;
		$description 	= $data->fulltext;
		$url 			= $data->url;
		$link 			= $data->org_url;
		$picture 		= $data->image_url;
		$mediaMode 		= $this->getMediaMode();

		$logger = AutotweetLogger::getInstance();
		$logger->log(JLog::INFO, 'sendFacebookMessage', $message);

		$fbId = $this->getFbChannelId();
		$accessToken = $this->get('fbchannel_access_token');
		$accessToken = $this->matchToken($accessToken, $fbId);

		// Post message and/or attachment
		switch ($mediaMode)
		{
			case 'attachment':
				$postAttachment = true;
				$postMessage = false;
				break;
			case 'both':
				$postMessage = true;
				$postAttachment = true;
				break;
			case 'message':
			default:
				$postMessage = true;
				$postAttachment = false;
		}

		$actions = array();
		$caption = null;

		if (empty($link))
		{
			$postAttachment = false;
		}
		else
		{
			// Extract data for action link
			$url_comps = parse_url($link);
			$caption = $url_comps['host'];

			if (EParameter::getComponentParam(CAUTOTWEETNG, 'include_authors'))
			{
				$author = $data->xtform->get('author');
				$author = JFactory::getUser($author);
				$authorName = $author->name;

				if (!empty($authorName))
				{
					$caption = $caption . ' | ' . $authorName;
				}
			}

			$actions['name'] = $caption;
			$actions['link'] = $link;
		}

		$name = TextUtil::truncString($name, self::MAX_CHARS_NAME);

		$arguments = array(
			'link' => $link,
			'name' => $name,
			'caption' => $caption,
			'description' => $description,
		);

		if (!empty($actions))
		{
			$arguments['actions'] = json_encode($actions);
		}

		if (strpos($link, '//www.youtube.com/') != false)
		{
			$source = str_replace('//www.youtube.com/watch?v=', '//www.youtube.com/v/', $link);
			$arguments['source'] = $source;

			if (empty($picture))
			{
				$picture = str_replace('//www.youtube.com/watch?v=', '//img.youtube.com/vi/', $link) . '/0.jpg';
			}
		}

		if ($isUserProfile)
		{
			$privacy = $this->get('sharedwith', 'EVERYONE');
			$privacy = array('value' => $privacy);
			$arguments['privacy'] = json_encode($privacy);
		}

		if ((empty($description)) && (empty($picture)))
		{
			$postAttachment = false;
		}

		// Include image tag only, when image url is not empty to avoid error "... must have a valid src..."
		if ( (!empty($picture)) && ($postAttachment) )
		{
			$arguments['picture'] = $picture;
		}

		// Message
		if ($postMessage)
		{
			$arguments['message'] = $message;
		}

		try
		{
			$post = $this->callApiPost('/' . $fbId . '/feed', $arguments, $accessToken);

			$msg = 'Facebook id: ' . $post['id'];
			$result = array(
				true,
				$msg
			);
		}
		catch (Exception $e)
		{
			$code = $e->getCode();
			$msg = $code . ' - ' . $e->getMessage();

			$result = array(
				false,
				$msg
			);
		}

		return $result;
	}
}
