<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2016 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */

defined('_JEXEC') or die;

/**
 * FbAppHelper class.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class FbAppHelper
{
	const TESTING_APP_ID = 'TXktQXBwLUlE';

	const PAGE_LIMIT = 256;

	private $facebook = null;

	protected $appId;

	protected $secret;

	protected $accessToken;

	/**
	 * FbAppHelper.
	 *
	 * @param   string  $app_id        Params
	 * @param   string  $secret        Params
	 * @param   string  $access_token  Params
	 */
	public function __construct($app_id, $secret, $access_token)
	{
		$this->appId = $app_id;
		$this->secret = $secret;
		$this->accessToken = $access_token;

		require_once JPATH_AUTOTWEET_VENDOR . '/autoload.php';
	}

	/**
	 * login.
	 *
	 * @param   bool  $force  Params
	 *
	 * @return	object
	 */
	public function login($force = false)
	{
		if ((!$this->facebook) || ($force))
		{
			$this->facebook = new Extly\Facebook\Facebook(
				[
					'app_id' => $this->appId,
					'app_secret' => $this->secret,
					'default_graph_version' => 'v2.2',
					'default_access_token' => $this->accessToken
				]
			);
		}

		return $this->facebook;
	}

	/**
	 * verify.
	 *
	 * @return	boolean
	 */
	public function verify()
	{
		$result = null;

		if (empty($this->accessToken))
		{
			$result = array(
				false,
				'Facebook Token not entered.'
			);

			return $result;
		}

		try
		{
			$response = $this->facebook->get('/me', $this->accessToken);
			$me = $response->getGraphUser();

			$result = array(
				true,
				JText::_('COM_AUTOTWEET_OK')
			);
		}
		catch (Extly\Facebook\Exceptions\FacebookResponseException $e)
		{
			$result = array(
				false,
				'Graph returned an error: ' . $e->getMessage()
			);
		}
		catch (Extly\Facebook\Exceptions\FacebookSDKException $e)
		{
			$result = array(
				false,
				'Facebook SDK returned an error: ' . $e->getMessage()
			);
		}
		catch (Exception $e)
		{
			$result = array(
				false,
				'Error: ' . $e->getMessage()
			);
		}

		return $result;
	}

	/**
	 * getUser.
	 *
	 * @param   string  $userId  Params.
	 *
	 * @return	object
	 */
	public function getUser($userId = null)
	{
		if ( (empty($userId)) || (!is_numeric($userId)) )
		{
			$user_id = 'me';
		}

		$response = $this->facebook->get(
			'/' . $user_id . '?fields=id,name,link',
			$this->accessToken
		);
		$user = $response->getGraphUser();

		return $user;
	}

	/**
	 * _getPagesAsChannel.
	 *
	 * @return	array
	 */
	private function _getPagesAsChannel()
	{
		$result = array();

		try
		{
			$response = $this->facebook->get(
				'/me/accounts?fields=id,name,link,access_token&amp;limit=' . self::PAGE_LIMIT,
				$this->accessToken
			);
			$items = $response->getGraphEdge();

			foreach ($items as $item)
			{
				$graphNode = $item->asArray();
				$result[] = array(
					'type' => 'Page',
					'id' => $graphNode['id'],
					'name' => $graphNode['name'],
					'url' => $graphNode['link'],
					'access_token' => $graphNode['access_token']
				);
			}
		}
		catch (Extly\Facebook\Exceptions\FacebookResponseException $e)
		{
			$result[] = array(
				'type' => 'Graph error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'Pages ' . $e->getMessage(),
				'token' => ''
			);
		}
		catch (Extly\Facebook\Exceptions\FacebookSDKException $e)
		{
			$result[] = array(
				'type' => 'Facebook SDK error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'Pages ' . $e->getMessage(),
				'token' => ''
			);
		}
		catch (Exception $e)
		{
			$result[] = array(
				'type' => 'Error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'Pages ' . $e->getMessage(),
				'token' => ''
			);
		}

		return $result;
	}

	/**
	 * _getGroupsAsChannel.
	 *
	 * @return	array
	 */
	private function _getGroupsAsChannel()
	{
		$result = array();

		try
		{
			$response = $this->facebook->get(
				'/me/groups?fields=id,name&amp;limit=' . self::PAGE_LIMIT,
				$this->accessToken
			);
			$items = $response->getGraphEdge();

			foreach ($items as $item)
			{
				$graphNode = $item->asArray();
				$url = 'https://www.facebook.com/' . $graphNode['id'];

				$result[] = array(
					'type' => 'Group',
					'id' => $graphNode['id'],
					'name' => $graphNode['name'],
					'url' => $url,
					'access_token' => $this->accessToken
				);
			}
		}
		catch (Extly\Facebook\Exceptions\FacebookResponseException $e)
		{
			$result[] = array(
				'type' => 'Graph error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'Group ' . $e->getMessage(),
				'token' => ''
			);
		}
		catch (Extly\Facebook\Exceptions\FacebookSDKException $e)
		{
			$result[] = array(
				'type' => 'Facebook SDK error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'Group ' . $e->getMessage(),
				'token' => ''
			);
		}
		catch (Exception $e)
		{
			$result[] = array(
				'type' => 'Error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'Group ' . $e->getMessage(),
				'token' => ''
			);
		}

		return $result;
	}

	/**
	 * getUserChannel.
	 *
	 * @return	array
	 */
	public function getUserChannel()
	{
		$channel = $this->_getUserAsChannel();

		if (count($channel) == 1)
		{
			return $channel[0];
		}

		return null;
	}

	/**
	 * _getUserAsChannel.
	 *
	 * @return	array
	 */
	private function _getUserAsChannel()
	{
		$result = array();

		try
		{
			$user = $this->getUser();

			if ($user)
			{
				$result[] = array(
					'type' => 'User',
					'id' => $user->getId(),
					'name' => $user->getName(),
					'url' => $user->getLink(),
					'access_token' => $this->accessToken
				);
			}
		}
		catch (Extly\Facebook\Exceptions\FacebookResponseException $e)
		{
			$result[] = array(
				'type' => 'Graph error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'User ' . $e->getMessage(),
				'token' => ''
			);
		}
		catch (Extly\Facebook\Exceptions\FacebookSDKException $e)
		{
			$result[] = array(
				'type' => 'Facebook SDK error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'User ' . $e->getMessage(),
				'token' => ''
			);
		}
		catch (Exception $e)
		{
			$result[] = array(
				'type' => 'Error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'User ' . $e->getMessage(),
				'token' => ''
			);
		}

		return $result;
	}

	/**
	 * getChannels.
	 *
	 * @return	array
	 */
	public function getChannels()
	{
		$user = $this->_getUserAsChannel();
		$pages = $this->_getPagesAsChannel();
		$groups = $this->_getGroupsAsChannel();

		return array_merge($user, $pages, $groups);
	}

	/**
	 * getPageChannels.
	 *
	 * @return	array
	 */
	public function getPageChannels()
	{
		$pages = $this->_getPagesAsChannel();

		return $pages;
	}

	/**
	 * getAlbums.
	 *
	 * @param   string  $channelId  Params
	 *
	 * @return	array
	 */
	public function getAlbums($channelId)
	{
		$result = array();

		try
		{
			$response = $this->facebook->get(
				"/{$channelId}/albums?fields=id,name,can_upload&amp;limit=" . self::PAGE_LIMIT,
				$this->accessToken
			);
			$items = $response->getGraphEdge();

			$result[] = array(
				'type' => 'Album',
				'id' => 0,
				'name' => ' - Default - '
			);

			foreach ($items as $item)
			{
				$graphNode = $item->asArray();

				if ($graphNode['can_upload'])
				{
					$a = array(
						'type' => 'Album',
						'id' => $graphNode['id'],
						'name' => $graphNode['name']
					);
					$result[] = $a;
				}
			}
		}
		catch (Extly\Facebook\Exceptions\FacebookResponseException $e)
		{
			$result[] = array(
				'type' => 'Graph error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'Album ' . $e->getMessage(),
				'token' => ''
			);
		}
		catch (Extly\Facebook\Exceptions\FacebookSDKException $e)
		{
			$result[] = array(
				'type' => 'Facebook SDK error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'Album ' . $e->getMessage(),
				'token' => ''
			);
		}
		catch (Exception $e)
		{
			$result[] = array(
				'type' => 'Error: ' . $e->getMessage(),
				'id' => '0',
				'name' => 'Album ' . $e->getMessage(),
				'token' => ''
			);
		}

		return $result;
	}

	/**
	 * getDebugToken.
	 *
	 * @param   string  $fbAccessToken  Params
	 *
	 * @return	array
	 */
	public function getDebugToken($fbAccessToken = null)
	{
		if ($this->appId == self::TESTING_APP_ID)
		{
			$response = array();
			$response['issued_at'] = '---';
			$response['expires_at'] = '---';

			return $response;
		}

		if (!$fbAccessToken)
		{
			$fbAccessToken = $this->accessToken;
		}

		try
		{
			$oAuth2Client = $this->facebook->getOAuth2Client();
			$tokenMetadata = $oAuth2Client->debugToken($this->accessToken);
			$tokenMetadata->validateAppId($this->appId);
			$tokenMetadata->validateExpiration();

			$issuedAt = $tokenMetadata->getIssuedAt();
			$timestamp = null;

			if ($issuedAt)
			{
				$timestamp = $issuedAt->getTimestamp();
			}

			$issuedAt = new JDate($timestamp);
			$response['issued_at'] = JHtml::_(
				'date',
				$issuedAt,
				JText::_('COM_AUTOTWEET_DATE_FORMAT')
			);

			$expiresAt = $tokenMetadata->getExpiresAt();

			if ($expiresAt)
			{
				$expiresAt = new JDate($tokenMetadata->getExpiresAt()->getTimestamp());

				$response['expires_at'] = JHtml::_(
					$expiresAt,
					JText::_('COM_AUTOTWEET_DATE_FORMAT')
				);
			}
			else
			{
				$response['expires_at'] = JText::_('COM_AUTOTWEET_VIEW_FBWACCOUNT_NEVER');
			}
		}
		catch (Exception $e)
		{
			$response = array();
			$response['issued_at'] = '------';
			$response['expires_at'] = '------';
		}

		return $response;
	}

	/**
	 * getExtendedAccessToken.
	 *
	 * @return	string
	 */
	public function getExtendedAccessToken()
	{
		try
		{
			$oAuth2Client = $this->facebook->getOAuth2Client();
			$tokenMetadata = $oAuth2Client->debugToken($this->accessToken);
			$tokenMetadata->validateAppId($this->appId);
			$tokenMetadata->validateExpiration();
			$longLivedAccessToken = $oAuth2Client->getLongLivedAccessToken($this->accessToken);

			return $longLivedAccessToken->getValue();
		}
		catch (Exception $e)
		{
			return null;
		}
	}
}
