<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Medium, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2016 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */

defined('_JEXEC') or die;

/**
 * MediumChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class MediumChannelHelper extends ChannelHelper
{
	protected $mediumClient = null;

	protected $integration_token = null;

	protected $is_auth = null;

	protected $me = null;

	/**
	 * ChannelHelper.
	 *
	 * @param   object  $channel   Params.
	 * @param   string  $integrationToken  Params.
	 */
	public function __construct($channel, $integrationToken = null)
	{
		parent::__construct($channel);

		if ($channel->id)
		{
			$this->integration_token = $this->channel->params->get('integration_token');
		}

		if ($integrationToken)
		{
			$this->integration_token = $integrationToken;
		}

		require_once JPATH_AUTOTWEET_VENDOR . '/autoload.php';
	}

	/**
	 * Internal service functions
	 *
	 * @return	object
	 */
	protected function getApiInstance()
	{
		if (!$this->mediumClient)
		{
			$this->mediumClient = new Extly\JonathanTorres\MediumSdk\Medium($this->integration_token);
		}

		return $this->mediumClient;
	}

	/**
	 * isAuth()
	 *
	 * @return	bool
	 */
	public function isAuth()
	{
		if (empty($this->integration_token))
		{
			$this->integration_token = null;

			return false;
		}

		try
		{
			$this->getApiInstance();

			$user = $this->mediumClient->getAuthenticatedUser();

			if ($user->data)
			{
				return $user;
			}
			else
			{
				return false;
			}
		}
		catch (Exception $e)
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());

			// Just in case, it is shown someday
			JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');

			// Invalidating access_token
			$ch->setToken($this->channel->id, 'integration_token', '');
		}

		return false;
	}

	/**
	 * sendMessage
	 *
	 * @param   string  $message  Param
	 * @param   array   $data     Param
	 *
	 * @return	array
	 */
	public function sendMessage($message, $data)
	{
		$logger = AutotweetLogger::getInstance();
		$logger->log(JLog::INFO, 'sendMediumMessage', $message);

		$user = $this->isAuth();

		if (!$user)
		{
			return array(
				false,
				JText::_('COM_AUTOTWEET_CHANNEL_MEDIUM_NOT_AUTH_ERR')
			);
		}

		$result = array(false, 'Medium Unknown Error', null);

		try
		{
			$this->getApiInstance();
			$content = $this->renderPost($this->channel->id, 'mediumchannel', $message, $data);

			$request = [
				'title' => $data->title,
				'contentFormat' => 'html',
				'content' => $content,
				'publishStatus' => 'public',
				'canonicalUrl' => $data->org_url
			];
			$post = $this->mediumClient->createPost($user->data->id, $request);

			$messageId = $post->data->id;
			$url = $post->data->url;

			$result = array(
				true
			);

			$result[] = 'OK - ' . $messageId . ' - ' . $url;
		}
		catch (Exception $e)
		{
			return array(
				false,
				$e->getMessage()
			);
		}

		return $result;
	}

	/**
	 * includeHashTags
	 *
	 * @return  bool
	 */
	public function includeHashTags()
	{
		return $this->channel->params->get('hashtags', true);
	}

	/**
	 * getSocialUrl
	 *
	 * @param   object  $user  Param
	 *
	 * @return	string
	 */
	public function getSocialUrl($user)
	{
		return $user->data->url;
	}
}
