<?php

namespace Extly\Http\Discovery\Strategy;

use Extly\GuzzleHttp\Psr7\Request as GuzzleRequest;
use Extly\Http\Message\MessageFactory\GuzzleMessageFactory;
use Extly\Http\Message\StreamFactory\GuzzleStreamFactory;
use Extly\Http\Message\UriFactory\GuzzleUriFactory;
use Extly\Http\Message\MessageFactory\DiactorosMessageFactory;
use Extly\Http\Message\StreamFactory\DiactorosStreamFactory;
use Extly\Http\Message\UriFactory\DiactorosUriFactory;
use Zend\Diactoros\Request as DiactorosRequest;
use Extly\Http\Message\MessageFactory\SlimMessageFactory;
use Extly\Http\Message\StreamFactory\SlimStreamFactory;
use Extly\Http\Message\UriFactory\SlimUriFactory;
use Slim\Http\Request as SlimRequest;
use Extly\Http\Adapter\Guzzle6\Client as Guzzle6;
use Extly\Http\Adapter\Guzzle5\Client as Guzzle5;
use Extly\Http\Client\Curl\Client as Curl;
use Extly\Http\Client\Socket\Client as Socket;
use Extly\Http\Adapter\React\Client as React;
use Extly\Http\Adapter\Buzz\Client as Buzz;

/**
 * @internal
 *
 * @author Tobias Nyholm <tobias.nyholm@gmail.com>
 */
final class CommonClassesStrategy implements DiscoveryStrategy
{
    /**
     * @var array
     */
    private static $classes = [
        'Extly\Http\Message\MessageFactory' => [
            ['class' => GuzzleMessageFactory::class, 'condition' => [GuzzleRequest::class, GuzzleMessageFactory::class]],
            ['class' => DiactorosMessageFactory::class, 'condition' => [DiactorosRequest::class, DiactorosMessageFactory::class]],
            ['class' => SlimMessageFactory::class, 'condition' => [SlimRequest::class, SlimMessageFactory::class]],
        ],
        'Extly\Http\Message\StreamFactory' => [
            ['class' => GuzzleStreamFactory::class, 'condition' => [GuzzleRequest::class, GuzzleStreamFactory::class]],
            ['class' => DiactorosStreamFactory::class, 'condition' => [DiactorosRequest::class, DiactorosStreamFactory::class]],
            ['class' => SlimStreamFactory::class, 'condition' => [SlimRequest::class, SlimStreamFactory::class]],
        ],
        'Extly\Http\Message\UriFactory' => [
            ['class' => GuzzleUriFactory::class, 'condition' => [GuzzleRequest::class, GuzzleUriFactory::class]],
            ['class' => DiactorosUriFactory::class, 'condition' => [DiactorosRequest::class, DiactorosUriFactory::class]],
            ['class' => SlimUriFactory::class, 'condition' => [SlimRequest::class, SlimUriFactory::class]],
        ],
        'Extly\Http\Client\HttpAsyncClient' => [
            ['class' => Guzzle6::class, 'condition' => Guzzle6::class],
            ['class' => Curl::class, 'condition' => Curl::class],
            ['class' => React::class, 'condition' => React::class],
        ],
        'Extly\Http\Client\HttpClient' => [
            ['class' => Guzzle6::class, 'condition' => Guzzle6::class],
            ['class' => Guzzle5::class, 'condition' => Guzzle5::class],
            ['class' => Curl::class, 'condition' => Curl::class],
            ['class' => Socket::class, 'condition' => Socket::class],
            ['class' => Buzz::class, 'condition' => Buzz::class],
            ['class' => React::class, 'condition' => React::class],
        ],
    ];

    /**
     * {@inheritdoc}
     */
    public static function getCandidates($type)
    {
        if (isset(self::$classes[$type])) {
            return self::$classes[$type];
        }

        return [];
    }
}
